@@ -1,48 +1,19 @@
 #!/usr/local/bin/perl -w
 #
-# rawtextFreq.pl version 0.07
-# (Updated 11/25/2003 -- Jason)
+# rawtextFreq.pl version 0.09
+# (Updated 05/17/2004 -- Jason)
 #
 # This program reads raw text and computes the frequency counts
-# for each synset in WordNet. These frequency counts are used by 
-# various measures of semantic relatedness to calculate the information 
-# content values of concepts. The output is generated in a format as 
-# required by the WordNet::Similarity modules (ver 0.04) for computing
+# for each synset in WordNet. These frequency counts are used by
+# various measures of semantic relatedness to calculate the information
+# content values of concepts. The output is generated in a format as
+# required by the WordNet::Similarity modules for computing
 # semantic relatedness.
-#
-# Copyright (c) 2002-2003
-#
-# Ted Pedersen, University of Minnesota, Duluth
-# tpederse at d.umn.edu
-#
-# Satanjeev Banerjee, Carnegie Mellon University, Pittsburgh
-# banerjee+ at cs.cmu.edu
-#
-# Siddharth Patwardhan, University of Utah, Salt Lake City
-# sidd at cs.utah.edu
-#
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# as published by the Free Software Foundation; either version 2
-# of the License, or (at your option) any later version.
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-#
-#
-#-----------------------------------------------------------------------------
+use strict;
+
+use File::Find;
 
 # Variable declarations
-my $wn;
-my $wnver;
-my $line;
-my $sentence;
-my @parts;
 my %compounds;
 my %stopWords;
 my %offsetFreq;
@@ -53,23 +24,34 @@ my %topHash;
 use Getopt::Long;
 use WordNet::QueryData;
 
+
 # First check if no commandline options have been provided... in which case
 # print out the usage notes!
-if ( $#ARGV == -1 )
+if ($#ARGV == -1)
 {
     &minimalUsageNotes();
-    exit;
+    exit 1;
 }
 
+our @opt_infiles;
+our ($opt_version, $opt_help, $opt_compfile, $opt_stopfile, $opt_outfile);
+our ($opt_wnpath, $opt_resnik, $opt_smooth, $opt_stdin);
+
 # Now get the options!
-&GetOptions("version", "help", "compfile=s", "stopfile=s", "outfile=s", "wnpath=s", "resnik", "smooth=s");
+my $ok = GetOptions("version", "help", "compfile=s", "stopfile=s", "outfile=s",
+		    "wnpath=s", "resnik", "smooth=s", "stdin",
+		    "infile=s" => \@opt_infiles);
+
+# GetOptions should have already printed out a detail error message if
+# $ok is false
+$ok or die "Error getting command-line arguments\n";
 
 # If the version information has been requested
 if(defined $opt_version)
 {
     $opt_version = 1;
     &printVersion();
-    exit;
+    exit 0;
 }
 
 # If detailed help has been requested
@@ -77,16 +59,29 @@ if(defined $opt_help)
 {
     $opt_help = 1;
     &printHelp();
-    exit;
+    exit 0;
 }
 
 # Look for the Compounds file... exit if not specified.
 if(!defined $opt_compfile)
 {
     &minimalUsageNotes();
-    exit;
+    exit 1;
 }
 
+# make sure either --stdin or --infile was given
+unless ($opt_stdin or scalar @opt_infiles) {
+    minimalUsageNotes ();
+    exit 1;
+}
+
+# make sure that both --stdin and --infile were NOT given
+if ($opt_stdin and scalar @opt_infiles) {
+    minimalUsageNotes ();
+    exit 1;
+}
+
+my $outfile;
 if(defined $opt_outfile)
 {
     $outfile = $opt_outfile;
@@ -94,12 +89,12 @@ if(defined $opt_outfile)
 else
 {
     &minimalUsageNotes();
-    exit;
+    exit 1;
 }
 
 # Load the compounds
 print STDERR "Loading compounds... ";
-open (WORDS, "$opt_compfile") || die ("Couldnt open $opt_compfile.\n");
+open (WORDS, '<', "$opt_compfile") or die ("Couldn't open $opt_compfile.\n");
 while (<WORDS>)
 {
     s/[\r\f\n]//g;
@@ -112,7 +107,7 @@ print STDERR "done.\n";
 if(defined $opt_stopfile)
 {
     print STDERR "Loading stoplist... ";
-    open (WORDS, "$opt_stopfile") || die ("Couldnt open $opt_stopfile.\n");
+    open (WORDS, '<', "$opt_stopfile") or die "Couldn't open $opt_stopfile.\n";
     while (<WORDS>)
     {
 	s/[\r\f\n]//g;
@@ -122,6 +117,8 @@ if(defined $opt_stopfile)
     print STDERR "done.\n";
 }
 
+my ($wnPCPath, $wnUnixPath);
+
 # Get the path to WordNet...
 if(defined $opt_wnpath)
 {
@@ -130,15 +127,25 @@ if(defined $opt_wnpath)
 }
 else
 {
-    $wnPCPath = (defined $ENV{"WNHOME"}) ? $ENV{"WNHOME"} : "C:\\Program Files\\WordNet\\1.7.1";
-    $wnUnixPath = (defined $ENV{"WNHOME"}) ? $ENV{"WNHOME"} : "/usr/local/WordNet-1.7.1";
-    $wnPCPath = (defined $ENV{"WNSEARCHDIR"}) ? $ENV{"WNSEARCHDIR"} : $wnPCPath."\\dict";
-    $wnUnixPath = (defined $ENV{"WNSEARCHDIR"}) ? $ENV{"WNSEARCHDIR"} : $wnUnixPath."/dict";
+    $wnPCPath = defined $ENV{WNHOME}
+                ? $ENV{WNHOME}
+	        : "C:\\Program Files\\WordNet\\2.0";
+    $wnUnixPath = defined $ENV{WNHOME}
+                  ? $ENV{WNHOME}
+                  : "/usr/local/WordNet-2.0";
+    $wnPCPath = defined $ENV{WNSEARCHDIR}
+                ? $ENV{WNSEARCHDIR}
+                : $wnPCPath."\\dict";
+    $wnUnixPath = defined $ENV{WNSEARCHDIR}
+                  ? $ENV{WNSEARCHDIR}
+                  : $wnUnixPath."/dict";
 }
 
 # Load up WordNet
 print STDERR "Loading WordNet... ";
-$wn = (defined $opt_wnpath) ? (WordNet::QueryData->new($opt_wnpath)) : (WordNet::QueryData->new());
+my $wn = (defined $opt_wnpath)
+         ? (WordNet::QueryData->new($opt_wnpath))
+         : (WordNet::QueryData->new());
 die "Unable to create WordNet object.\n" if(!$wn);
 $wnPCPath = $wnUnixPath = $wn->dataPath() if(defined $wn->can('dataPath'));
 print STDERR "done.\n";
@@ -149,25 +156,68 @@ print STDERR "Loading topmost nodes of the hierarchies... ";
 print STDERR "done.\n";
 
 # Read the input, form sentences and process each
-print STDERR "Computing frequencies... ";
-$sentence = "";
-while($line=<>)
-{
-    $line=~s/[\r\f\n]//g;
-    @parts = split(/[.?!]/, $line);
-    foreach (1..$#parts)
-    {
-	$sentence .= shift(@parts)." ";
-	&process($sentence);
-	$sentence = "";
+#  We need to process each sentence (or clause really) at the same time.
+#  This is because process() tries to find compounds in the sentence.  It is
+#  not sufficient to process a line at a time because a sentence often
+#  spans more than one line; therefore, a compound could also span a line
+#  break (for example, the first word of a compound could be the last word
+#  on a line, and the second word of the compound could be the first word
+#  on the next line).
+my $sentence = "";
+# check if we're reading from a file or from STDIN
+if (scalar @opt_infiles) {
+
+    # first we have to figure out what files to process.  The files
+    # are specified with in --infile option.  The value of the option
+    # can be a filename, a directory, or a pattern (as understood by
+    # Perl's glob() function).
+    my @infiles = getFiles (@opt_infiles);
+
+    print STDERR "Computing frequencies.\n";
+    foreach my $i (0..$#infiles) {
+	my $infile = $infiles[$i];
+
+	print STDERR ("  Processing '$infile' (", $i + 1, "/",
+		      $#infiles + 1, " files)... ");
+	
+	open (IFH, '<', $infile) or die "Cannot open file '$infile': $!";
+	while (my $line = <IFH>)
+	{
+	    $line =~ s/[\r\f\n]//g;
+	    my @parts = split (/[.?!,;:]/, $line);
+	    foreach (1..$#parts)
+	    {
+		$sentence .= shift(@parts)." ";
+		process ($sentence);
+		$sentence = "";
+	    }
+	    $sentence .= shift(@parts)." " if(@parts);
+	}
+	process ($sentence);
+	close IFH or die "Cannot not close file '$infile': $!";
+	print STDERR "  done.\n";
     }
-    $sentence .= shift(@parts)." " if(@parts);
 }
-&process($sentence);
+else {
+    print STDERR "Computing frequencies... ";
+    while (my $line = <STDIN>)
+    {
+	$line =~ s/[\r\f\n]//g;
+	my @parts = split (/[.?!,;:]/, $line);
+	foreach (1..$#parts)
+	{
+	    $sentence .= shift (@parts) . " ";
+	    process ($sentence);
+	    $sentence = "";
+	}
+	$sentence .= shift (@parts) . " " if @parts;
+    }
+    process ($sentence);
+}
 print STDERR "done.\n";
 
 # Hack to prevent warning...
-$opt_resnik = 1 if(defined $opt_resnik);
+$opt_resnik = 1 if defined $opt_resnik;
 
 # Smoothing!
 if(defined $opt_smooth)
@@ -175,7 +225,7 @@ if(defined $opt_smooth)
     print STDERR "Smoothing... ";
     if($opt_smooth eq 'ADD1')
     {
-	foreach $pos ("noun", "verb")
+	foreach my $pos ("noun", "verb")
 	{
 	    my $localpos = $pos;
 
@@ -192,7 +242,7 @@ if(defined $opt_smooth)
 	    {
 		last if(/^\S/);
 	    }
-	    ($offset) = split(/\s+/, $_, 2);
+	    my ($offset) = split(/\s+/, $_, 2);
 	    $offset =~ s/^0*//;
 	    $offsetFreq{$localpos}{$offset}++;
 	    while(<IDX>)
@@ -213,8 +263,9 @@ if(defined $opt_smooth)
     }
 }
 
-# Propagating frequencies up the WordNet hierarchies... 
+# Propagating frequencies up the WordNet hierarchies...
 print STDERR "Propagating frequencies up through WordNet... ";
+
 $offsetFreq{"n"}{0} = 0;
 $offsetFreq{"v"}{0} = 0;
 &propagateFrequency(0, "n");
@@ -225,11 +276,11 @@ print STDERR "done.\n";
 
 # Print the output to file
 print STDERR "Writing output file... ";
-open(OUT, ">$outfile") || die "Unable to open $outfile for writing.\n";
+open(OUT, ">$outfile") || die "Unable to open $outfile for writing: $!\n";
 print OUT "wnver::".$wn->version()."\n";
-foreach $pos ("n", "v")
+foreach my $pos ("n", "v")
 {
-    foreach $offset (sort {$a <=> $b} keys %{$newFreq{$pos}})
+    foreach my $offset (sort {$a <=> $b} keys %{$newFreq{$pos}})
     {
 	print OUT "$offset$pos $newFreq{$pos}{$offset}";
 	print OUT " ROOT" if($topHash{$pos}{$offset});
@@ -239,6 +290,7 @@ foreach $pos ("n", "v")
 close(OUT);
 print STDERR "done.\n";
 
+
 # ----------------- Subroutines start Here ----------------------
 
 # Processing of each sentence
@@ -251,7 +303,7 @@ print STDERR "done.\n";
 sub process
 {
     my $block;
-    
+
     $block = lc(shift);
     $block =~ s/\'//g;
     $block =~ s/[^a-z0-9]+/ /g;
@@ -259,6 +311,7 @@ sub process
     $block =~ s/^\s+//;
     $block =~ s/\s+$//;
     $block = &compoundify($block);
+
     while($block =~ /([\w_]+)/g)
     {
 	&updateFrequency($1) if(!defined $stopWords{$1});
@@ -268,28 +321,18 @@ sub process
 # Form all possible compounds within a sentence
 sub compoundify
 {
-    my $block;
-    my $string;
+    my $block = shift; # get the block of text
     my $done;
     my $temp;
-    my $firstPointer;
     my $secondPointer;
-    my @wordsArray;
-    
-    # get the block of text
-    $block = shift;
-    
+
     # get all the words into an array
-    @wordsArray = ();
-    while ($block =~ /(\w+)/g)
-    {
-	push @wordsArray, $1;
-    }
-    
+    my @wordsArray = $block =~ /(\w+)/g;
+
     # now compoundify, GREEDILY!!
-    $firstPointer = 0;
-    $string = "";
-    
+    my $firstPointer = 0;
+    my $string = "";
+
     while($firstPointer <= $#wordsArray)
     {
 	$secondPointer = $#wordsArray;
@@ -299,26 +342,26 @@ sub compoundify
 	    $temp = join ("_", @wordsArray[$firstPointer..$secondPointer]);
 	    if(exists $compounds{$temp})
 	    {
-		$string .= "$temp "; 
+		$string .= "$temp ";
 		$done = 1;
 	    }
-	    else 
-	    { 
-		$secondPointer--; 
+	    else
+	    {
+		$secondPointer--;
 	    }
 	}
-	if(!$done) 
-	{ 
-	    $string .= "$wordsArray[$firstPointer] "; 
+	if(!$done)
+	{
+	    $string .= "$wordsArray[$firstPointer] ";
 	}
 	$firstPointer = $secondPointer + 1;
     }
     $string =~ s/ $//;
-    
-    return $string;    
+
+    return $string;
 }
 
-# Subroutine to update frequency tokens on "seeing" a 
+# Subroutine to update frequency tokens on "seeing" a
 # word in text
 sub updateFrequency
 {
@@ -331,7 +374,6 @@ sub updateFrequency
     $word = shift;
     foreach $pos ("n", "v")
     {
-	@senses = $wn->querySense($word."\#".$pos);
 	@forms = $wn->validForms($word."\#".$pos);
 	foreach $form (@forms)
 	{
@@ -375,8 +417,13 @@ sub propagateFrequency
     }
     else
     {
-	$newFreq{$pos}{$node} = ($offsetFreq{$pos}{$node})?$offsetFreq{$pos}{$node}:0;
-	return ($offsetFreq{$pos}{$node})?$offsetFreq{$pos}{$node}:0;
+	$newFreq{$pos}{$node} = $offsetFreq{$pos}{$node}
+	                        ? $offsetFreq{$pos}{$node}
+                                : 0;
+
+	return $offsetFreq{$pos}{$node}
+	       ? $offsetFreq{$pos}{$node}
+               : 0;
     }
     $sum = 0;
     if($#{$retValue} >= 0)
@@ -386,8 +433,13 @@ sub propagateFrequency
 	    $sum += &propagateFrequency($hyponym, $pos);
 	}
     }
-    $newFreq{$pos}{$node} = (($offsetFreq{$pos}{$node})?$offsetFreq{$pos}{$node}:0) + $sum;
-    return (($offsetFreq{$pos}{$node})?$offsetFreq{$pos}{$node}:0) + $sum;
+    $newFreq{$pos}{$node} = ($offsetFreq{$pos}{$node}
+			     ? $offsetFreq{$pos}{$node}
+			     : 0) + $sum;
+
+    return ($offsetFreq{$pos}{$node}
+	    ? $offsetFreq{$pos}{$node}
+	    : 0) + $sum;
 }
 
 # Subroutine that returns the hyponyms of a given synset.
@@ -400,7 +452,7 @@ sub getHyponymOffsets
     my @retVal;
 
     $offset = shift;
-    $pos = shift;
+    my $pos = shift;
     if($offset == 0)
     {
 	@retVal = keys %{$topHash{$pos}};
@@ -424,44 +476,84 @@ sub getHyponymOffsets
 # Creates and loads the topmost nodes hash.
 sub createTopHash
 {
-    my $word;
-    my $wps;
-    my $upper;
-    my $fileIsGood;
-    my %wpsOffset;
+    my $datapath = $wn->dataPath;
+    my $unixfile_n = "${datapath}/data.noun";
+    my $windozefile_n = "${datapath}\\noun.dat";
 
-    undef %wpsOffset;
-    foreach $word ($wn->listAllWords("n"))
-    {
-	foreach $wps ($wn->querySense($word."\#n"))
-	{
-	    if(!$wpsOffset{$wn->offset($wps)})
-	    {
-		($upper) = $wn->querySense($wps, "hype");
-		if(!$upper)
-		{
-		    $topHash{"n"}{$wn->offset($wps)} = 1;	
+    my $nounfile = -e $windozefile_n ? $windozefile_n : $unixfile_n;
+
+    open (NFH, '<', $nounfile) or die "Cannot open '$nounfile': $!";
+
+    while (<NFH>) {
+        next if "  " eq substr $_, 0, 2;
+	next if / \@ \d\d\d\d\d\d\d\d /;
+	next unless /^(\d\d\d\d\d\d\d\d) /;
+	my $offset = $1 + 0;
+
+	# QueryData::getSense will die() if the $offset and pos are not
+	# found.  Putting this in an eval will catch the exception.  See
+	# perldoc -f eval
+	my $wps;
+	eval {$wps = $wn->getSense ($offset, 'n')};
+	if ($@) {
+	    die "(offset '$offset' not found) $@";
+	}
+	$topHash{n}{$offset} = 1;
+    }
+
+    close NFH;
+
+    my $unixfile_v = "${datapath}/data.verb";
+    my $windozefile_v = "${datapath}\\verb.dat";
+
+    my $verbfile = -e $windozefile_v ? $windozefile_v : $unixfile_v;
+
+    open (VFH, '<', $verbfile) or die "Cannot open '$verbfile': $!";
+
+    while (<VFH>) {
+        next if " " eq substr ($_, 0, 2);
+	next if / \@ \d\d\d\d\d\d\d\d /;
+	next unless /^(\d\d\d\d\d\d\d\d) /;
+	my $offset = $1 + 0;
+	$topHash{v}{$offset} = 1;
+    }
+
+    close VFH;
+}
+
+sub getFiles
+{
+    my @inpatterns = @_;
+    my @infiles;
+    # the options to pass to File::Find::find()
+    my %options = (wanted =>
+		   sub {
+		       unless (-d $File::Find::name) {
+			   push @infiles, $File::Find::name;
+		       }
+		   },
+		   follow_fast => 1);
+
+    foreach my $pattern (@inpatterns) {
+	if (-d $pattern) {
+	    find (\%options, $pattern);
+	}
+	elsif (-e $pattern and not -d $pattern) {
+	    push @infiles, $pattern;
+	}
+	else {
+	    my @files = glob $pattern;
+	    foreach my $file (@files) {
+		if (-d $file) {
+		    find (\%options, $pattern);
 		}
-		$wpsOffset{$wn->offset($wps)} = 1;
-	    }
-	}
-    }
-    undef %wpsOffset;
-    foreach $word ($wn->listAllWords("v"))
-    {
-	foreach $wps ($wn->querySense($word."\#v"))
-	{
-	    if(!$wpsOffset{$wn->offset($wps)})
-	    {
-		($upper) = $wn->querySense($wps, "hype");
-		if(!$upper)
-		{
-		    $topHash{"v"}{$wn->offset($wps)} = 1;
+		else {
+		    push @infiles, $file;
 		}
-		$wpsOffset{$wn->offset($wps)} = 1;
 	    }
 	}
     }
+    return @infiles;
 }
 
 # Subroutine to print detailed help
@@ -474,6 +566,8 @@ sub printHelp
     print "--compfile       Used to specify the file COMPFILE containing the \n";
     print "                 list of compounds in WordNet.\n";
     print "--outfile        Specifies the output file OUTFILE.\n";
+    print "--stdin          Read the input from the standard input\n";
+    print "--infile         INFILE is the name of an input file\n";
     print "--stopfile       STOPFILE is a list of stop listed words that will\n";
     print "                 not be considered in the frequency count.\n";
     print "--wnpath         Option to specify WNPATH as the location of WordNet data\n";
@@ -483,7 +577,7 @@ sub printHelp
     print "--resnik         Option to specify that the frequency counting should\n";
     print "                 be performed according to the method described by\n";
     print "                 Resnik (1995).\n";
-    print "--smooth         Specifies the smoothing to be used on the probabilities\n"; 
+    print "--smooth         Specifies the smoothing to be used on the probabilities\n";
     print "                 computed. SCHEME specifies the type of smoothing to\n";
     print "                 perform. It is a string, which can be only be 'ADD1'\n";
     print "                 as of now. Other smoothing schemes will be added in\n";
@@ -502,27 +596,34 @@ sub minimalUsageNotes
 # Subroutine that prints the usage
 sub printUsage
 {
-    print "rawtextFreq.pl [{--compfile COMPFILE --outfile OUTFILE [--stopfile STOPFILE]";
-    print " [--wnpath WNPATH] [--resnik] [--smooth SCHEME] [FILE...] | --help | --version }]\n"
+    print <<'EOT';
+Usage: rawtextFreq.pl --compfile COMPFILE --outfile OUTFILE
+                       {--stdin | --infile FILE [--infile FILE ...]}
+                       [--stopfile FILE] [--resnik] [--wnpath PATH]
+                       [--smooth SCHEME]
+                      | --help | --version
+EOT
 }
 
 # Subroutine to print the version information
 sub printVersion
 {
-    print "rawtextFreq.pl version 0.07\n";
-    print "Copyright (c) 2002-2003 Ted Pedersen, Satanjeev Banerjee & Siddharth Patwardhan.\n";
+    print "rawtextFreq.pl version 0.09\n";
+    print "Copyright (C) 2002-2004 Ted Pedersen, Satanjeev Banerjee & Siddharth Patwardhan.\n";
 }
 
 __END__
 
 =head1 NAME
 
-rawtextFreq.pl
+rawtextFreq.pl - Perl program for finding the frequencies of words in raw text
+files
 
 =head1 SYNOPSIS
 
-rawtextFreq.pl [--compfile=COMPFILE --outfile=OUTFILE [--stopfile=STOPFILE] [--
-wnpath=WNPATH] [--resnik] [--smooth=SCHEME] FILES... | --help -- version]
+rawtextFreq.pl --compfile COMPFILE --outfile OUTFILE [--stopfile=STOPFILE]
+{--stdin | --infile FILE [--infile FILE ...]} [--wnpath WNPATH]
+[--resnik] [--smooth=SCHEME] | --help | --version
 
 =head1 OPTIONS
 
@@ -564,10 +665,62 @@ B<--version>
 
     Display version information
 
-B<FILES>
+B<--stdin>
+
+    Read from the standard input the text that is to be used for
+    counting the frequency of words.
+
+B<--infile>=I<PATTERN>
+
+    The name of a raw text file to be used to count word frequencies.
+    This can actually be a filename, a directory name, or a pattern (as
+    understood by Perl's glob() function).  If the value is a directory
+    name, then all the files in that directory and its subdirectories will
+    be used.
 
-    A list of raw text files to be used to count word frequencies.
     If you are looking for some interesting files to use, check out
     Project Gutenberg: <http://www.gutenberg.org>.
 
+    This option may be given more than once (if more than one file
+    should be used).
+
+=head1 AUTHORS
+
+ Siddharth Patwardhan, University of Utah, Salt Lake City
+ sidd @ cs.utah.edu
+
+ Ted Pedersen, University of Minnesota, Duluth
+ tpederse @ d.umn.edu
+
+ Satanjeev Banerjee, Carnegie Mellon University, Pittsburgh
+ banerjee+ @ cs.cmu.edu
+
+ Jason Michelizzi, University of Minnesota, Duluth
+ mich0212 @ d.umn.edu
+
+=head1 BUGS
+
+None.
+
+=head1 COPYRIGHT AND LICENSE
+
+Copyright (c) 2002-2004, Siddharth Patwardhan, Ted Pedersen, Satanjeev
+Banerjee, and Jason Michelizzi
+
+This program is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License
+as published by the Free Software Foundation; either version 2
+of the License, or (at your option) any later version.
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to
+
+ Free Software Foundation, Inc.
+ 59 Temple Place - Suite 330
+ Boston, MA  02111-1307, USA
+
 =cut
