@@ -1,16 +1,16 @@
 #!/usr/local/bin/perl -w
 #
-# semCorRawFreq.pl version 0.07
+# semCorRawFreq.pl version 0.09
 #
-# This program reads SemCor files and computes the frequency 
+# This program reads SemCor files and computes the frequency
 # counts for each synset in WordNet, ignoring the sense tags in the corpus
-# (treating it like a raw text corpus). These frequency counts are used by 
-# various measures of semantic relatedness to calculate the information 
-# content values of concepts. The output is generated in a format as 
-# required by the WordNet::Similarity modules (ver 0.01) for computing
-# semantic relatedness.
+# (treating it like a raw text corpus). These frequency counts are used by
+# various measures of semantic relatedness to calculate the information
+# content values of concepts. The output is generated in a format as
+# required by the WordNet::Similarity modules for computing semantic
+# relatedness.
 #
-# Copyright (c) 2002-2003
+# Copyright (C) 2002-2004
 # Ted Pedersen, University of Minnesota, Duluth
 # tpederse at d.umn.edu
 #
@@ -33,14 +33,13 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #
-#
 #-----------------------------------------------------------------------------
 
+use strict;
+
+use File::Find;
+
 # Variable declarations
-my $wn;
-my $wnver;
-my $line;
-my $sentence;
 my @parts;
 my %compounds;
 my %stopWords;
@@ -60,8 +59,18 @@ if($#ARGV == -1)
     exit;
 }
 
+our (@opt_infiles, $opt_version, $opt_help, $opt_compfile, $opt_stopfile);
+our ($opt_outfile, $opt_wnpath, $opt_resnik, $opt_smooth, $opt_stdin);
+
 # Now get the options!
-&GetOptions("version", "help", "compfile=s", "stopfile=s", "outfile=s", "wnpath=s", "resnik", "smooth=s");
+my $okay = GetOptions("version", "help", "compfile=s", "stopfile=s",
+		      "outfile=s", "wnpath=s", "resnik", "smooth=s",
+		      "stdin", "infile=s" => \@opt_infiles);
+
+unless ($okay) {
+    minimalUsageNotes ();
+    exit 1;
+}
 
 # If the version information has been requested
 if(defined $opt_version)
@@ -83,19 +92,22 @@ if(defined $opt_help)
 if(!defined $opt_compfile)
 {
     &minimalUsageNotes();
-    exit;
+    exit 1;
 }
 
-if(defined $opt_outfile)
+unless (defined $opt_outfile)
 {
-    $outfile = $opt_outfile;
-}
-else
-{
-    &minimalUsageNotes();
-    exit;
+    minimalUsageNotes();
+    exit 1;
 }
 
+unless ((defined $opt_stdin or scalar @opt_infiles)
+	and not (defined $opt_stdin and scalar @opt_infiles)) {
+    minimalUsageNotes ();
+    exit 1;
+}
+my $wnPCPath;
+my $wnUnixPath;
 # Get the path to WordNet...
 if(defined $opt_wnpath)
 {
@@ -104,15 +116,15 @@ if(defined $opt_wnpath)
 }
 else
 {
-    $wnPCPath = (defined $ENV{"WNHOME"}) ? $ENV{"WNHOME"} : "C:\\Program Files\\WordNet\\1.7.1";
-    $wnUnixPath = (defined $ENV{"WNHOME"}) ? $ENV{"WNHOME"} : "/usr/local/WordNet-1.7.1";
+    $wnPCPath = (defined $ENV{"WNHOME"}) ? $ENV{"WNHOME"} : "C:\\Program Files\\WordNet\\2.0";
+    $wnUnixPath = (defined $ENV{"WNHOME"}) ? $ENV{"WNHOME"} : "/usr/local/WordNet-2.0";
     $wnPCPath = (defined $ENV{"WNSEARCHDIR"}) ? $ENV{"WNSEARCHDIR"} : $wnPCPath."\\dict";
     $wnUnixPath = (defined $ENV{"WNSEARCHDIR"}) ? $ENV{"WNSEARCHDIR"} : $wnUnixPath."/dict";
 }
 
 # Load the compounds
 print STDERR "Loading compounds... ";
-open (WORDS, "$opt_compfile") || die ("Couldnt open $opt_compfile.\n");
+open (WORDS, "$opt_compfile") or die ("Couldn't open $opt_compfile.\n");
 while (<WORDS>)
 {
     s/[\r\f\n]//g;
@@ -122,13 +134,13 @@ close WORDS;
 print STDERR "done.\n";
 
 # Hack to prevent warning...
-$opt_resnik = 1 if(defined $opt_resnik);
+$opt_resnik = 1 if defined $opt_resnik;
 
 # Load the stop words if specified
 if(defined $opt_stopfile)
 {
     print STDERR "Loading stoplist... ";
-    open (WORDS, "$opt_stopfile") || die ("Couldnt open $opt_stopfile.\n");
+    open (WORDS, "$opt_stopfile") || die ("Couldn't open $opt_stopfile.\n");
     while (<WORDS>)
     {
 	s/[\r\f\n]//g;
@@ -140,7 +152,9 @@ if(defined $opt_stopfile)
 
 # Load up WordNet
 print STDERR "Loading WordNet... ";
-$wn=(defined $opt_wnpath)? (WordNet::QueryData->new($opt_wnpath)):(WordNet::QueryData->new());
+my $wn = defined $opt_wnpath
+         ? (WordNet::QueryData->new($opt_wnpath))
+         : (WordNet::QueryData->new());
 die "Unable to create WordNet object.\n" if(!$wn);
 $wnPCPath = $wnUnixPath = $wn->dataPath() if($wn->can('dataPath'));
 print STDERR "done.\n";
@@ -151,19 +165,51 @@ print STDERR "Loading topmost nodes of the hierarchies... ";
 print STDERR "done.\n";
 
 # Read the input, form sentences and process each
-print STDERR "Computing frequencies... ";
-$sentence = "";
-while($line=<>)
-{
-    $line=~s/[\r\f\n]//g;
-    while($line =~ /<wf([^>]+)>/g)
+if (scalar @opt_infiles) {
+    # first we have to figure out what files to process.  The files
+    # are specified with in --infile option.  The value of the option
+    # can be a filename, a directory, or a pattern (as understood by
+    # Perl's glob() function).
+    my @infiles = getFiles (@opt_infiles);
+
+    print STDERR "Computing frequencies... ";
+    foreach my $i (0..$#infiles) {
+	my $infile = $infiles[$i];
+
+	print STDERR ("  Processing '$infile' (", $i + 1,
+		      "/", scalar @infiles, " files)... ");
+
+	open (IFH, '<', $infile) or die "Cannot open '$infile': $!";
+	while (my $line = <IFH>) {
+	    $line =~ s/[\r\f\n]+//g;
+	    while ($line =~ /<wf([^>]+)>/g) {
+		my $tagAttribs = $1;
+		if ($tagAttribs =~ /cmd=done/) {
+		    if ($tagAttribs =~ /lemma=([^ ]+) /) {
+			updateFrequency ($1) unless defined $stopWords{$1};
+		    }
+		}
+	    }
+	}
+	close IFH or die "Cannot close file: $!";
+	print STDERR "done.\n";
+    }
+}
+else {
+    print STDERR "Computing frequencies... ";
+    my $sentence = "";
+    while(my $line = <STDIN>)
     {
-	$tagAttribs = $1;
-	if($tagAttribs =~ /cmd=done/)
+	$line =~ s/[\r\f\n]+//g;
+	while($line =~ /<wf([^>]+)>/g)
 	{
-	    if($tagAttribs =~ /lemma=([^ ]+) /)
+	    my $tagAttribs = $1;
+	    if($tagAttribs =~ /cmd=done/)
 	    {
-		&updateFrequency($1) if(!defined $stopWords{$1});
+		if($tagAttribs =~ /lemma=([^ ]+) /)
+		{
+		    &updateFrequency($1) if(!defined $stopWords{$1});
+		}
 	    }
 	}
     }
@@ -176,7 +222,7 @@ if(defined $opt_smooth)
     print STDERR "Smoothing... ";
     if($opt_smooth eq 'ADD1')
     {
-	foreach $pos ("noun", "verb")
+	foreach my $pos ("noun", "verb")
 	{
 	    my $localpos = $pos;
 
@@ -193,7 +239,7 @@ if(defined $opt_smooth)
 	    {
 		last if(/^\S/);
 	    }
-	    ($offset) = split(/\s+/, $_, 2);
+	    my ($offset) = split(/\s+/, $_, 2);
 	    $offset =~ s/^0*//;
 	    $offsetFreq{$localpos}{$offset}++;
 	    while(<IDX>)
@@ -226,17 +272,20 @@ print STDERR "done.\n";
 
 # Print the output to file
 print STDERR "Writing output file... ";
-open(OUT, ">$outfile") || die "Unable to open $outfile for writing.\n";
+open(OUT, '>', $opt_outfile)
+    or die "Unable to open '$opt_outfile' for writing.";
+
 print OUT "wnver::".$wn->version()."\n";
-foreach $pos ("n", "v")
+foreach my $pos ("n", "v")
 {
-    foreach $offset (sort {$a <=> $b} keys %{$newFreq{$pos}})
+    foreach my $offset (sort {$a <=> $b} keys %{$newFreq{$pos}})
     {
 	print OUT "$offset$pos $newFreq{$pos}{$offset}";
 	print OUT " ROOT" if($topHash{$pos}{$offset});
 	print OUT "\n";
     }
 }
+
 close(OUT);
 print "done.\n";
 
@@ -255,7 +304,6 @@ sub updateFrequency
     $word = shift;
     foreach $pos ("n", "v")
     {
-	@senses = $wn->querySense($word."\#".$pos);
 	@forms = $wn->validForms($word."\#".$pos);
 	foreach $form (@forms)
 	{
@@ -317,27 +365,22 @@ sub propagateFrequency
 # Subroutine that returns the hyponyms of a given synset.
 sub getHyponymOffsets
 {
-    my $offset;
-    my $wordForm;
-    my $hyponym;
-    my @hyponyms;
     my @retVal;
-
-    $offset = shift;
-    $pos = shift;
+    my $offset = shift;
+    my $pos = shift;
     if($offset == 0)
     {
 	@retVal = keys %{$topHash{$pos}};
 	return [@retVal];
     }
-    $wordForm = $wn->getSense($offset, $pos);
-    @hyponyms = $wn->querySense($wordForm, "hypo");
+    my $wordForm = $wn->getSense($offset, $pos);
+    my @hyponyms = $wn->querySense($wordForm, "hypo");
     if(!@hyponyms || $#hyponyms < 0)
     {
 	return undef;
     }
     @retVal = ();
-    foreach $hyponym (@hyponyms)
+    foreach my $hyponym (@hyponyms)
     {
 	$offset = $wn->offset($hyponym);
 	push @retVal, $offset;
@@ -388,6 +431,41 @@ sub createTopHash
     }
 }
 
+sub getFiles
+{
+    my @inpatterns = @_;
+    my @infiles;
+    # the options to pass to File::Find::find()
+    my %options = (wanted =>
+		   sub {
+		       unless (-d $File::Find::name) {
+			   push @infiles, $File::Find::name;
+		       }
+		   },
+		   follow_fast => 1);
+
+    foreach my $pattern (@inpatterns) {
+	if (-d $pattern) {
+	    find (\%options, $pattern);
+	}
+	elsif (-e $pattern and not -d $pattern) {
+	    push @infiles, $pattern;
+	}
+	else {
+	    my @files = glob $pattern;
+	    foreach my $file (@files) {
+		if (-d $file) {
+		    find (\%options, $pattern);
+		}
+		else {
+		    push @infiles, $file;
+		}
+	    }
+	}
+    }
+    return @infiles;
+}
+
 # Subroutine to print detailed help
 sub printHelp
 {
@@ -398,6 +476,8 @@ sub printHelp
     print "--compfile       Used to specify the file COMPFILE containing the \n";
     print "                 list of compounds in WordNet.\n";
     print "--outfile        Specifies the output file OUTFILE.\n";
+    print "--stdin          Read input from the standard input.\n";
+    print "--infile         Specifies the name of an input file.\n";
     print "--stopfile       STOPFILE is a list of stop listed words that will\n";
     print "                 not be considered in the frequency count.\n";
     print "--wnpath         Option to specify WNPATH as the location of WordNet data\n";
@@ -419,20 +499,25 @@ sub printHelp
 # Subroutine to print minimal usage notes
 sub minimalUsageNotes
 {
-    &printUsage();
+    printUsage ();
     print "Type semCorRawFreq.pl --help for detailed help.\n";
 }
 
 # Subroutine that prints the usage
 sub printUsage
 {
-    print "semCorRawFreq.pl [{--compfile COMPFILE --outfile OUTFILE [--stopfile STOPFILE]";
-    print " [--wnpath WNPATH] [--resnik] [--smooth SCHEME] [FILE...] | --help | --version }]\n"
+    print <<'EOU';
+Usage: semCorRawFreq.pl --compfile FILE --outfile FILE
+                         {--stdin | --infile FILE [--infile FILE ...]}
+                         [--stopfile FILE] [--resnik] [--wnpath PATH]
+                         [--smooth SCHEME]
+                        | --help | --version
+EOU
 }
 
 # Subroutine to print the version information
 sub printVersion
 {
-    print "semCorRawFreq.pl version 0.07\n";
-    print "Copyright (c) 2002-2003 Ted Pedersen, Satanjeev Banerjee & Siddharth Patwardhan.\n";
+    print "semCorRawFreq.pl version 0.09\n";
+    print "Copyright (c) 2002-2004 Ted Pedersen, Satanjeev Banerjee & Siddharth Patwardhan.\n";
 }
